<?php

namespace FluentForm\App\Modules\Form;

use FluentForm\App\Services\Browser\Browser;
use FluentForm\App\Modules\ReCaptcha\ReCaptcha;
use FluentForm\Framework\Foundation\Application;
use FluentForm\Framework\Helpers\ArrayHelper as Arr;
use FluentForm\App\Services\FormBuilder\MessageShortCodeParser;

class FormHandler
{
    /**
     * @var \FluentForm\Framework\Foundation\Application
     */
    protected $app;

    /**
     * @var \FluentForm\Framework\Request\Request
     */
    protected $request;

    /**
     * @var array $formData
     */
    protected $formData;

    /**
     * The fluent form object.
     *
     * @var \stdClass
     */
    protected $form;

    /**
     * Form Handler constructor.
     *
     * @param \FluentForm\Framework\Foundation\Application $app
     */
    public function __construct(Application $app)
    {
        $this->app = $app;
        $this->request = $app->request;
    }

    /**
     * Set the form using it's ID.
     *
     * @param $formId
     * @return $this
     */
    public function setForm($formId)
    {
        $this->form = wpFluent()->table('fluentform_forms')->find($formId);

        return $this;
    }

    /**
     * Handle form submition
     */
    public function onSubmit()
    {
        // Parse the url encoded data from the request object.
        parse_str($this->app->request->get('data'), $data);

        // Merge it back again to the request object.
        $this->app->request->merge(['data' => $data]);

        $formId = intval($this->app->request->get('form_id'));

        $this->setForm($formId);

        // Parse the form and get the flat inputs with validations.
        $fields = FormFieldsParser::getInputs($this->form, ['rules']);

        // Sanitize the data properly.
        $this->formData = fluentFormSanitizer($data, null, $fields);

        // Now validate the data using the previous validations.
        $this->validate($fields);

        // Prepare the data to be inserted to the DB.
        $insertData = $this->prepareInsertData();

        $insertId = wpFluent()->table('fluentform_submissions')->insert($insertData);

        try {
            $this->app->doAction('fluentform_submission_inserted', $insertId, $this->formData, $this->form);
        } catch (\Exception $e) {
            if (defined('WP_DEBUG') && WP_DEBUG) {
                return $e;
            }
        }

        $this->sendResponse($insertId);
    }

    /**
     * Prepare response and do actions/filters
     * and send the response to the client.
     *
     * @param int $insertId
     */
    private function sendResponse($insertId)
    {
        $formSettings = $this->form->settings;

        $confirmation = $this->form->settings['confirmation'];

        $message = MessageShortCodeParser::parseMessageShortCode(
            [['message' => $confirmation['messageToShow']]],
            $insertId,
            $this->formData,
            $this->form,
            false
        );

        if ($confirmation['redirectTo'] == 'samePage') {
            $returnData = [
                'message'      => $message[0]['message'],
                'formBehavior' => $confirmation['samePageFormBehavior'],
            ];
        } else {
            $redirectUrl = $confirmation['customUrl'];
            if ($confirmation['redirectTo'] == 'customPage') {
                $redirectUrl = get_permalink($confirmation['customPage']);
            }

            $returnData = [
                'message'     => $message[0]['message'],
                'redirectUrl' => $redirectUrl
            ];
        }

        $returnData = $this->app->applyFilters(
            'fluentform_submission_confirmation',
            $returnData,
            $this->form,
            $confirmation
        );

        wp_send_json_success([
            'insert_id' => $insertId,
            'result'    => $returnData
        ], 200);
    }

    /**
     * Validate form data.
     *
     * @param $fields
     * @return bool
     */
    private function validate(&$fields)
    {
        $this->validateRestrictions($fields);

        $this->validateNonce();

        $this->validateReCaptcha();

        $validations = FormFieldsParser::getValidations($this->form, $this->formData, $fields);

        // Fire an event so that one can hook into it to work with the rules & messages.
        $validations = $this->app->applyFilters('fluentform_validations', $validations, $this->form);

        $validator = \FluentValidator\Validator::make($this->formData, $validations[0], $validations[1]);

        if ($validator->validate()->fails()) {
            $errors = [];

            foreach ($validator->errors() as $attribute => $rules) {
                $position = strpos($attribute, ']');

                if ($position) {
                    $attribute = substr($attribute, 0, strpos($attribute, ']') + 1);
                }

                $errors[$attribute] = $rules;
            }

            // Fire an event so that one can hook into it to work with the errors.
            $errors = $this->app->applyFilters('fluentform_validation_error', $errors, $this->form);

            wp_send_json(['errors' => $errors], 422);
        }

        return true;
    }

    /**
     * Validate nonce.
     */
    protected function validateNonce()
    {
        $formId = $this->form->id;

        $shouldVerifyNonce = $this->app->applyFilters('fluentform_nonce_verify', true, $formId);

        if ($shouldVerifyNonce) {
            $nonce = Arr::get($this->formData, '_fluentform_'.$formId.'_fluentformnonce');
            if (!wp_verify_nonce($nonce, 'fluentform-submit-form')) {
                $errors = $this->app->applyFilters('fluentForm_nonce_error', [
                    '_fluentformnonce' => [
                        __('Nonce verification failed, please try again.', 'fluentform')
                    ]
                ]);

                wp_send_json(['errors' => $errors], 422);
            }
        }
    }

    /**
     * Validate reCaptcha.
     */
    private function validateReCaptcha()
    {
        if (FormFieldsParser::hasElement($this->form, 'recaptcha')) {
            $isValid = ReCaptcha::validate(Arr::get($this->formData, 'g-recaptcha-response'));

            if (!$isValid) {
                wp_send_json([
                    'errors' => [
                        'g-recaptcha-response' => [
                            __('reCaptcha verification failed, please try again.', 'fluentform')
                        ]
                    ]
                ], 422);
            }
        }
    }

    /**
     * Validate form data based on the form restrictions settings.
     *
     * @param $fields
     */
    private function validateRestrictions(&$fields)
    {
        $formSettings = wpFluent()->table('fluentform_form_meta')
                                  ->where('form_id', $this->form->id)
                                  ->where('meta_key', 'formSettings')
                                  ->first();

        $this->form->settings = $formSettings ? json_decode($formSettings->value, true) : [];

        $isAllowed = [
            'status'  => true,
            'message' => ''
        ];

        // This will check the following restriction settings.
        // 1. limitNumberOfEntries
        // 2. scheduleForm
        // 3. requireLogin
        $isAllowed = apply_filters('fluentform_is_form_renderable', $isAllowed, $this->form);

        if (!$isAllowed['status']) {
            wp_send_json([
                'errors' => [
                    'restricted' => [
                        __($isAllowed['message'], 'fluentform')
                    ]
                ]
            ], 422);
        }

        // Since we are here, we should now handle if the form should be allowed to submit empty.
        $restrictions = Arr::get($this->form->settings, 'restrictions.denyEmptySubmission', []);

        $this->handleDenyEmptySubmission($restrictions, $fields);
    }

    /**
     * Handle response when empty form submission is not allowed.
     *
     * @param array $settings
     * @param $fields
     */
    private function handleDenyEmptySubmission($settings = [], &$fields)
    {
        // Determine whether empty form submission is allowed or not.
        if (Arr::get($settings, 'enabled')) {
            // confirm this form has no required fields.
            if (!FormFieldsParser::hasRequiredFields($this->form, $fields)) {
                // Filter out the form data which doesn't have values.
                $filteredFormData = array_filter(
                // Filter out the other meta fields that aren't actual inputs.
                    array_intersect_key($this->formData, $fields)
                );

                if (!count($filteredFormData)) {
                    wp_send_json([
                        'errors' => [
                            'restricted' => [
                                __(Arr::get($settings, 'message'), 'fluentform')
                            ]
                        ]
                    ], 422);
                }
            }
        }
    }

    /**
     * Prepare the data to be inserted to the database.
     *
     * @param  boolean $formData
     * @return array
     */
    public function prepareInsertData($formData = false)
    {
        $formId = $this->form->id;

        if (!$formData) {
            $formData = $this->formData;
        }

        $previousItem = wpFluent()->table('fluentform_submissions')
                                  ->where('form_id', $formId)
                                  ->orderBy('id', 'DESC')
                                  ->first();

        $serialNumber = 1;

        if ($previousItem) {
            $serialNumber = $previousItem->serial_number + 1;
        }

        $browser = new Browser;

        $inputConfigs = FormFieldsParser::getEntryInputs($this->form);

        $this->formData = apply_filters('fluentform_insert_response_data', $formData, $formId, $inputConfigs);

        return [
            'form_id'       => $formId,
            'serial_number' => $serialNumber,
            'response'      => json_encode($this->formData),
            'source_url'    => site_url(Arr::get($this->formData, '_wp_http_referer')),
            'user_id'       => get_current_user_id(),
            'browser'       => $browser->getBrowser(),
            'device'        => $browser->getPlatform(),
            'ip'            => $this->app->request->getIp(),
            'created_at'    => date('Y-m-d H:i:s'),
            'updated_at'    => date('Y-m-d H:i:s')
        ];
    }

    /**
     * Delegate the validation rules & messages to the
     * ones that the validation library recognizes.
     *
     * @param  $rules
     * @param  $messages
     * @return array
     */
    protected function delegateValidations($rules, $messages, $search = [], $replace = [])
    {
        $search = $search ?: ['max_file_size', 'allowed_file_types'];
        $replace = $replace ?: ['max', 'mimes'];

        foreach ($rules as &$rule) {
            $rule = str_replace($search, $replace, $rule);
        }

        foreach ($messages as $key => $message) {
            $newKey = str_replace($search, $replace, $key);
            $messages[$newKey] = $message;
            unset($messages[$key]);
        }

        return [$rules, $messages];
    }
}
