<?php

namespace FluentForm\App\Modules\Integration\ActiveCampaign;

use FluentForm\Framework\Foundation\Application;
use FluentForm\App\Modules\Integration\BaseIntegration;

class Integrator extends BaseIntegration
{
    /**
     * Option `key` for Active Campaign List settings.
     *
     * @var string $key
     */
    private $key = 'activecampaign_details';

    /**
     * Application instance
     *
     * @var \FluentForm\Framework\Foundation\Application $app
     */
    private $app;

    /**
     * Global settings for the integration.
     *
     * @var array $settings
     */
    protected $settings;

    /**
     * Active Campaign integration constructor.
     *
     * @param \FluentForm\Framework\Foundation\Application $app
     */
    public function __construct(Application $app)
    {
        parent::__construct(
            $this->key,
            $app->request->get('form_id', false),
            true
        );

        $this->app = $app;
    }

    /**
     * Get all of the integrations.
     *
     * @return void
     */
    public function index()
    {
        $globalStatus = $this->isActive();

        $integrations = $this->getAll();

        wp_send_json_success([
            'global_status' => $globalStatus,
            'integrations'  => $integrations,
            'configure_url' => admin_url('admin.php?page=fluent_forms_settings#activeCampaign')
        ], 200);
    }

    /**
     * Determine if the integration is active.
     *
     * @return boolean
     */
    private function isActive()
    {
        $this->settings = get_option('_fluentform_' . $this->key);

        return $this->settings && $this->settings['status'];
    }

    /**
     * Get all available lists.
     *
     * @return void
     */
    public function getLists()
    {
        if (!$this->isActive()) {
            wp_send_json_error([
                'error' => __('ActiveCampaign is not configured yet', 'fluentform')
            ], 400);
        }

        try {
            $activeCampaignApi = new \ActiveCampaign($this->settings['apiUrl'], $this->settings['apiKey']);

            $lists = $activeCampaignApi->api('list/list', ['ids' => 'all']);

            if (!$lists->success) {
                throw new \Exception(__('No lists found.', 'fluentform'));
            }
        } catch (\Exception $exception) {
            wp_send_json_error([
                'message' => $exception->getMessage()
            ], 400);
        }

        wp_send_json_success([
            'lists' => $this->formatLists($lists)
        ], 200);
    }

    /**
     * Format the lists excluding some keys so that we only have
     * the plain array of the lists containing each list info.
     *
     * @param array $lists
     * @return array
     */
    private function formatLists($lists)
    {
        $excludeKeys = [
            'result_code',
            'result_message',
            'result_output',
            'http_code',
            'success',
            'error',
        ];

        return array_values(array_diff_key((array) $lists, array_flip($excludeKeys)));
    }
}
