<?php

namespace FluentForm\App\Services\FormBuilder\Components;

use FluentForm\App;

class BaseComponent
{
    /**
     * $app Application Instance
     * @var Framework\Foundation\Application
     */
	protected $app = null;
	
    /**
     * Construct the base component object
     */
	public function __construct()
	{
		$this->app = App::make();
	}

    /**
     * Build unique ID concating form id and name attribute
     * @param  array $data $form
     * @return string for id value
     */
	protected function makeElementId($data, $form) {
	    if(isset($data['attributes']['name'])) {
            return "ff_{$form->id}_{$data['attributes']['name']}";
        }
    }

    /**
     * Build attributes for any html element
     * @param  array $attributes
     * @return string [Compiled key='value' attributes]
     */
	protected function buildAttributes($attributes, $form = null)
	{
		$atts = '';
		foreach ($attributes as $key => $value) {
			if($value) {
                if($key == 'value') {
                    $value = apply_filters(
                        'fluentform_parse_default_value', $value, $form
                    );
                }

				$atts .= "{$key}='{$value}' ";
			}
		}
		return $atts;
	}

    /**
     * Extract value attribute from attribute list
     * @param  array &$element
     * @return string
     */
	protected function extractValueFromAttributes(&$element)
	{
		$value = $element['attributes']['value'];
		unset($element['attributes']['value']);
		return $value;
	}

    /**
     * Determine if the given element has conditions bound
     * @param  array $element [Html element being compiled]
     * @return boolean
     */
	protected function hasConditions($element)
	{
		$conditionals = @$element['settings']['conditional_logics'];
		return (@$conditionals['status'] && @$conditionals['conditions']);
	}

    /**
     * Generate a unique id for an element
     * @param  string $str [preix]
     * @return string [Unique id]
     */
	protected function getUniqueId($str)
	{
		return $str.'_'.md5(uniqid(mt_rand(), true));
	}

    /**
     * Get a default class for each form element wrapper
     * @return string
     */
	protected function getDefaultContainerClass()
	{
		return 'ff-el-group ';
	}

    /**
     * Get required class for form element wrapper
     * @param  array $rules [Validation rules]
     * @return mixed
     */
	protected function getRequiredClass($rules)
    {
	    $required = $rules['required'];
        if (isset($required)) {
            return $required['value'] ? 'ff-el-is-required ' : '';
        }
    }

    /**
     * Generate a label for any element
     * @param  array $data
     * @return string [label Html element]
     */
    protected function buildElementLabel($data) {
        $markup = "<div class='%s'>
            <label for='%s'>%s</label>
        </div>";

        $html = sprintf($markup,
            trim('ff-el-input--label ' . $this->getRequiredClass($data['settings']['validation_rules'])),
            $data['attributes']['id'],
            $data['settings']['label']
        );
        return $html;
    }

    /**
     * Generate html/markup for any element
     * @param  string $elMarkup [Predifined partial markup]
     * @param  array $data
     * @param  StdClass $form [Form object]
     * @return string [Compiled markup]
     */
    protected function buildElementMarkup($elMarkup, $data, $form)
    {
        $hasConditions = $this->hasConditions($data) ? 'has-conditions ' : '';
        
        $labelPlacementClass = isset($data['settings']['label_placement'])
            ? 'ff-el-form-' . $data['settings']['label_placement'] . ' '
            : '';

        $labelClass = trim(
            'ff-el-input--label ' .
            $this->getRequiredClass($data['settings']['validation_rules'])
        );

        $formGroupClass = trim(
            $this->getDefaultContainerClass().
            $labelPlacementClass.
            $hasConditions.
            @$data['settings']['container_class']
        );

        $labelHelpText = $inputHelpText = '';
        if($form->settings['layout']['helpMessagePlacement'] == 'with_label') {
            $labelHelpText = $this->getLabelHelpMessage($data);
        } else {
            $inputHelpText = $this->getInputHelpMessage($data);
        }

        $forStr = '';
        if (isset($data['attributes']['id'])) {
            $forStr = "for='{$data['attributes']['id']}'";
        }

        $labelMarkup = '';
        if (!empty($data['settings']['label'])) {
            $labelMarkup = "<div class='%s'><label %s>%s</label> {$labelHelpText}</div>";
        }

        $markup = "<div class='%s'>
            {$labelMarkup}
            <div class='ff-el-input--content'>
                {$elMarkup}{$inputHelpText}
            </div>
        </div>";

        $html = sprintf($markup,
            $formGroupClass,
            $labelClass,
            $forStr,
            @$data['settings']['label']
        );

        return $html;
    }

    /**
     * Generate a help message for any element beside label
     * @param  array $data
     * @return string [Html]
     */
    protected function getLabelHelpMessage($data)
    {
        if(isset($data['settings']['help_message']) && $data['settings']['help_message'] != '') {
            return "<div class='ff-el-tooltip' data-content='{$data['settings']['help_message']}'>
                <i class='icon-info-circle'></i>
            </div>";
        }
    }

    /**
     * Generate a help message for any element beside form element
     * @param  array $data
     * @return string [Html]
     */
    protected function getInputHelpMessage($data)
    {
        if (isset($data['settings']['help_message'])) {
            return "<div class='ff-el-help-message'>{$data['settings']['help_message']}</div>";
        }
        return false;
    }
}
